# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Optional
from typing_extensions import Literal

import httpx

from ...types import (
    WhatsappBusinessProfileVertical,
    sender_list_params,
    sender_create_params,
    sender_update_params,
    sender_update_profile_params,
    sender_upload_profile_picture_params,
)
from ..._types import Body, Omit, Query, Headers, NoneType, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .agent.agent import (
    AgentResource,
    AsyncAgentResource,
    AgentResourceWithRawResponse,
    AsyncAgentResourceWithRawResponse,
    AgentResourceWithStreamingResponse,
    AsyncAgentResourceWithStreamingResponse,
)
from ...pagination import SyncCursor, AsyncCursor
from ..._base_client import AsyncPaginator, make_request_options
from ...types.sender import Sender
from ...types.webhook_event import WebhookEvent
from ...types.webhook_secret_response import WebhookSecretResponse
from ...types.sender_update_profile_response import SenderUpdateProfileResponse
from ...types.whatsapp_business_profile_response import WhatsappBusinessProfileResponse
from ...types.whatsapp_business_profile_vertical import WhatsappBusinessProfileVertical
from ...types.sender_upload_profile_picture_response import SenderUploadProfilePictureResponse

__all__ = ["SendersResource", "AsyncSendersResource"]


class SendersResource(SyncAPIResource):
    @cached_property
    def agent(self) -> AgentResource:
        return AgentResource(self._client)

    @cached_property
    def with_raw_response(self) -> SendersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/zavudev/sdk-python#accessing-raw-response-data-eg-headers
        """
        return SendersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SendersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/zavudev/sdk-python#with_streaming_response
        """
        return SendersResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        name: str,
        phone_number: str,
        set_as_default: bool | Omit = omit,
        webhook_events: List[WebhookEvent] | Omit = omit,
        webhook_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Sender:
        """
        Create sender

        Args:
          webhook_events: Events to subscribe to.

          webhook_url: HTTPS URL for webhook events.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/senders",
            body=maybe_transform(
                {
                    "name": name,
                    "phone_number": phone_number,
                    "set_as_default": set_as_default,
                    "webhook_events": webhook_events,
                    "webhook_url": webhook_url,
                },
                sender_create_params.SenderCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Sender,
        )

    def retrieve(
        self,
        sender_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Sender:
        """
        Get sender

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        return self._get(
            f"/v1/senders/{sender_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Sender,
        )

    def update(
        self,
        sender_id: str,
        *,
        email_receiving_enabled: bool | Omit = omit,
        name: str | Omit = omit,
        set_as_default: bool | Omit = omit,
        webhook_active: bool | Omit = omit,
        webhook_events: List[WebhookEvent] | Omit = omit,
        webhook_url: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Sender:
        """
        Update sender

        Args:
          email_receiving_enabled: Enable or disable inbound email receiving for this sender.

          webhook_active: Whether the webhook is active.

          webhook_events: Events to subscribe to.

          webhook_url: HTTPS URL for webhook events. Set to null to remove webhook.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        return self._patch(
            f"/v1/senders/{sender_id}",
            body=maybe_transform(
                {
                    "email_receiving_enabled": email_receiving_enabled,
                    "name": name,
                    "set_as_default": set_as_default,
                    "webhook_active": webhook_active,
                    "webhook_events": webhook_events,
                    "webhook_url": webhook_url,
                },
                sender_update_params.SenderUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Sender,
        )

    def list(
        self,
        *,
        cursor: str | Omit = omit,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursor[Sender]:
        """
        List senders

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/senders",
            page=SyncCursor[Sender],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "limit": limit,
                    },
                    sender_list_params.SenderListParams,
                ),
            ),
            model=Sender,
        )

    def delete(
        self,
        sender_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete sender

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/v1/senders/{sender_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def get_profile(
        self,
        sender_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WhatsappBusinessProfileResponse:
        """Get the WhatsApp Business profile for a sender.

        The sender must have a WhatsApp
        Business Account connected.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        return self._get(
            f"/v1/senders/{sender_id}/profile",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WhatsappBusinessProfileResponse,
        )

    def regenerate_webhook_secret(
        self,
        sender_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WebhookSecretResponse:
        """Regenerate the webhook secret for a sender.

        The old secret will be invalidated
        immediately.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        return self._post(
            f"/v1/senders/{sender_id}/webhook/secret",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WebhookSecretResponse,
        )

    def update_profile(
        self,
        sender_id: str,
        *,
        about: str | Omit = omit,
        address: str | Omit = omit,
        description: str | Omit = omit,
        email: str | Omit = omit,
        vertical: WhatsappBusinessProfileVertical | Omit = omit,
        websites: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SenderUpdateProfileResponse:
        """Update the WhatsApp Business profile for a sender.

        The sender must have a
        WhatsApp Business Account connected.

        Args:
          about: Short description of the business (max 139 characters).

          address: Physical address of the business (max 256 characters).

          description: Extended description of the business (max 512 characters).

          email: Business email address.

          vertical: Business category for WhatsApp Business profile.

          websites: Business website URLs (maximum 2).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        return self._patch(
            f"/v1/senders/{sender_id}/profile",
            body=maybe_transform(
                {
                    "about": about,
                    "address": address,
                    "description": description,
                    "email": email,
                    "vertical": vertical,
                    "websites": websites,
                },
                sender_update_profile_params.SenderUpdateProfileParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SenderUpdateProfileResponse,
        )

    def upload_profile_picture(
        self,
        sender_id: str,
        *,
        image_url: str,
        mime_type: Literal["image/jpeg", "image/png"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SenderUploadProfilePictureResponse:
        """Upload a new profile picture for the WhatsApp Business profile.

        The image will
        be uploaded to Meta and set as the profile picture.

        Args:
          image_url: URL of the image to upload.

          mime_type: MIME type of the image.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        return self._post(
            f"/v1/senders/{sender_id}/profile/picture",
            body=maybe_transform(
                {
                    "image_url": image_url,
                    "mime_type": mime_type,
                },
                sender_upload_profile_picture_params.SenderUploadProfilePictureParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SenderUploadProfilePictureResponse,
        )


class AsyncSendersResource(AsyncAPIResource):
    @cached_property
    def agent(self) -> AsyncAgentResource:
        return AsyncAgentResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncSendersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/zavudev/sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSendersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSendersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/zavudev/sdk-python#with_streaming_response
        """
        return AsyncSendersResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        name: str,
        phone_number: str,
        set_as_default: bool | Omit = omit,
        webhook_events: List[WebhookEvent] | Omit = omit,
        webhook_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Sender:
        """
        Create sender

        Args:
          webhook_events: Events to subscribe to.

          webhook_url: HTTPS URL for webhook events.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/senders",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "phone_number": phone_number,
                    "set_as_default": set_as_default,
                    "webhook_events": webhook_events,
                    "webhook_url": webhook_url,
                },
                sender_create_params.SenderCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Sender,
        )

    async def retrieve(
        self,
        sender_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Sender:
        """
        Get sender

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        return await self._get(
            f"/v1/senders/{sender_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Sender,
        )

    async def update(
        self,
        sender_id: str,
        *,
        email_receiving_enabled: bool | Omit = omit,
        name: str | Omit = omit,
        set_as_default: bool | Omit = omit,
        webhook_active: bool | Omit = omit,
        webhook_events: List[WebhookEvent] | Omit = omit,
        webhook_url: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Sender:
        """
        Update sender

        Args:
          email_receiving_enabled: Enable or disable inbound email receiving for this sender.

          webhook_active: Whether the webhook is active.

          webhook_events: Events to subscribe to.

          webhook_url: HTTPS URL for webhook events. Set to null to remove webhook.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        return await self._patch(
            f"/v1/senders/{sender_id}",
            body=await async_maybe_transform(
                {
                    "email_receiving_enabled": email_receiving_enabled,
                    "name": name,
                    "set_as_default": set_as_default,
                    "webhook_active": webhook_active,
                    "webhook_events": webhook_events,
                    "webhook_url": webhook_url,
                },
                sender_update_params.SenderUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Sender,
        )

    def list(
        self,
        *,
        cursor: str | Omit = omit,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Sender, AsyncCursor[Sender]]:
        """
        List senders

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/senders",
            page=AsyncCursor[Sender],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "limit": limit,
                    },
                    sender_list_params.SenderListParams,
                ),
            ),
            model=Sender,
        )

    async def delete(
        self,
        sender_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete sender

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/v1/senders/{sender_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def get_profile(
        self,
        sender_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WhatsappBusinessProfileResponse:
        """Get the WhatsApp Business profile for a sender.

        The sender must have a WhatsApp
        Business Account connected.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        return await self._get(
            f"/v1/senders/{sender_id}/profile",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WhatsappBusinessProfileResponse,
        )

    async def regenerate_webhook_secret(
        self,
        sender_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WebhookSecretResponse:
        """Regenerate the webhook secret for a sender.

        The old secret will be invalidated
        immediately.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        return await self._post(
            f"/v1/senders/{sender_id}/webhook/secret",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WebhookSecretResponse,
        )

    async def update_profile(
        self,
        sender_id: str,
        *,
        about: str | Omit = omit,
        address: str | Omit = omit,
        description: str | Omit = omit,
        email: str | Omit = omit,
        vertical: WhatsappBusinessProfileVertical | Omit = omit,
        websites: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SenderUpdateProfileResponse:
        """Update the WhatsApp Business profile for a sender.

        The sender must have a
        WhatsApp Business Account connected.

        Args:
          about: Short description of the business (max 139 characters).

          address: Physical address of the business (max 256 characters).

          description: Extended description of the business (max 512 characters).

          email: Business email address.

          vertical: Business category for WhatsApp Business profile.

          websites: Business website URLs (maximum 2).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        return await self._patch(
            f"/v1/senders/{sender_id}/profile",
            body=await async_maybe_transform(
                {
                    "about": about,
                    "address": address,
                    "description": description,
                    "email": email,
                    "vertical": vertical,
                    "websites": websites,
                },
                sender_update_profile_params.SenderUpdateProfileParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SenderUpdateProfileResponse,
        )

    async def upload_profile_picture(
        self,
        sender_id: str,
        *,
        image_url: str,
        mime_type: Literal["image/jpeg", "image/png"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SenderUploadProfilePictureResponse:
        """Upload a new profile picture for the WhatsApp Business profile.

        The image will
        be uploaded to Meta and set as the profile picture.

        Args:
          image_url: URL of the image to upload.

          mime_type: MIME type of the image.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        return await self._post(
            f"/v1/senders/{sender_id}/profile/picture",
            body=await async_maybe_transform(
                {
                    "image_url": image_url,
                    "mime_type": mime_type,
                },
                sender_upload_profile_picture_params.SenderUploadProfilePictureParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SenderUploadProfilePictureResponse,
        )


class SendersResourceWithRawResponse:
    def __init__(self, senders: SendersResource) -> None:
        self._senders = senders

        self.create = to_raw_response_wrapper(
            senders.create,
        )
        self.retrieve = to_raw_response_wrapper(
            senders.retrieve,
        )
        self.update = to_raw_response_wrapper(
            senders.update,
        )
        self.list = to_raw_response_wrapper(
            senders.list,
        )
        self.delete = to_raw_response_wrapper(
            senders.delete,
        )
        self.get_profile = to_raw_response_wrapper(
            senders.get_profile,
        )
        self.regenerate_webhook_secret = to_raw_response_wrapper(
            senders.regenerate_webhook_secret,
        )
        self.update_profile = to_raw_response_wrapper(
            senders.update_profile,
        )
        self.upload_profile_picture = to_raw_response_wrapper(
            senders.upload_profile_picture,
        )

    @cached_property
    def agent(self) -> AgentResourceWithRawResponse:
        return AgentResourceWithRawResponse(self._senders.agent)


class AsyncSendersResourceWithRawResponse:
    def __init__(self, senders: AsyncSendersResource) -> None:
        self._senders = senders

        self.create = async_to_raw_response_wrapper(
            senders.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            senders.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            senders.update,
        )
        self.list = async_to_raw_response_wrapper(
            senders.list,
        )
        self.delete = async_to_raw_response_wrapper(
            senders.delete,
        )
        self.get_profile = async_to_raw_response_wrapper(
            senders.get_profile,
        )
        self.regenerate_webhook_secret = async_to_raw_response_wrapper(
            senders.regenerate_webhook_secret,
        )
        self.update_profile = async_to_raw_response_wrapper(
            senders.update_profile,
        )
        self.upload_profile_picture = async_to_raw_response_wrapper(
            senders.upload_profile_picture,
        )

    @cached_property
    def agent(self) -> AsyncAgentResourceWithRawResponse:
        return AsyncAgentResourceWithRawResponse(self._senders.agent)


class SendersResourceWithStreamingResponse:
    def __init__(self, senders: SendersResource) -> None:
        self._senders = senders

        self.create = to_streamed_response_wrapper(
            senders.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            senders.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            senders.update,
        )
        self.list = to_streamed_response_wrapper(
            senders.list,
        )
        self.delete = to_streamed_response_wrapper(
            senders.delete,
        )
        self.get_profile = to_streamed_response_wrapper(
            senders.get_profile,
        )
        self.regenerate_webhook_secret = to_streamed_response_wrapper(
            senders.regenerate_webhook_secret,
        )
        self.update_profile = to_streamed_response_wrapper(
            senders.update_profile,
        )
        self.upload_profile_picture = to_streamed_response_wrapper(
            senders.upload_profile_picture,
        )

    @cached_property
    def agent(self) -> AgentResourceWithStreamingResponse:
        return AgentResourceWithStreamingResponse(self._senders.agent)


class AsyncSendersResourceWithStreamingResponse:
    def __init__(self, senders: AsyncSendersResource) -> None:
        self._senders = senders

        self.create = async_to_streamed_response_wrapper(
            senders.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            senders.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            senders.update,
        )
        self.list = async_to_streamed_response_wrapper(
            senders.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            senders.delete,
        )
        self.get_profile = async_to_streamed_response_wrapper(
            senders.get_profile,
        )
        self.regenerate_webhook_secret = async_to_streamed_response_wrapper(
            senders.regenerate_webhook_secret,
        )
        self.update_profile = async_to_streamed_response_wrapper(
            senders.update_profile,
        )
        self.upload_profile_picture = async_to_streamed_response_wrapper(
            senders.upload_profile_picture,
        )

    @cached_property
    def agent(self) -> AsyncAgentResourceWithStreamingResponse:
        return AsyncAgentResourceWithStreamingResponse(self._senders.agent)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["PhoneNumberPricing"]


class PhoneNumberPricing(BaseModel):
    is_free_eligible: Optional[bool] = FieldInfo(alias="isFreeEligible", default=None)
    """Whether this number qualifies for the free first US number offer."""

    monthly_price: Optional[float] = FieldInfo(alias="monthlyPrice", default=None)
    """Monthly price in USD."""

    upfront_price: Optional[float] = FieldInfo(alias="upfrontPrice", default=None)
    """One-time purchase price in USD."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["MessageContent", "Button", "Contact", "Section", "SectionRow"]


class Button(BaseModel):
    id: str

    title: str


class Contact(BaseModel):
    name: Optional[str] = None

    phones: Optional[List[str]] = None


class SectionRow(BaseModel):
    id: str

    title: str

    description: Optional[str] = None


class Section(BaseModel):
    rows: List[SectionRow]

    title: str


class MessageContent(BaseModel):
    """Content for non-text message types (WhatsApp and Telegram)."""

    buttons: Optional[List[Button]] = None
    """Interactive buttons (max 3)."""

    contacts: Optional[List[Contact]] = None
    """Contact cards for contact messages."""

    emoji: Optional[str] = None
    """Emoji for reaction messages."""

    filename: Optional[str] = None
    """Filename for documents."""

    latitude: Optional[float] = None
    """Latitude for location messages."""

    list_button: Optional[str] = FieldInfo(alias="listButton", default=None)
    """Button text for list messages."""

    location_address: Optional[str] = FieldInfo(alias="locationAddress", default=None)
    """Address of the location."""

    location_name: Optional[str] = FieldInfo(alias="locationName", default=None)
    """Name of the location."""

    longitude: Optional[float] = None
    """Longitude for location messages."""

    media_id: Optional[str] = FieldInfo(alias="mediaId", default=None)
    """WhatsApp media ID if already uploaded."""

    media_url: Optional[str] = FieldInfo(alias="mediaUrl", default=None)
    """URL of the media file (for image, video, audio, document, sticker)."""

    mime_type: Optional[str] = FieldInfo(alias="mimeType", default=None)
    """MIME type of the media."""

    react_to_message_id: Optional[str] = FieldInfo(alias="reactToMessageId", default=None)
    """Message ID to react to."""

    sections: Optional[List[Section]] = None
    """Sections for list messages."""

    template_id: Optional[str] = FieldInfo(alias="templateId", default=None)
    """Template ID for template messages."""

    template_variables: Optional[Dict[str, str]] = FieldInfo(alias="templateVariables", default=None)
    """Variables for template rendering. Keys are variable positions (1, 2, 3...)."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from zavudev import Zavudev, AsyncZavudev
from tests.utils import assert_matches_type
from zavudev.pagination import SyncCursor, AsyncCursor
from zavudev.types.senders.agent import (
    AgentKnowledgeBase,
    KnowledgeBaseCreateResponse,
    KnowledgeBaseUpdateResponse,
    KnowledgeBaseRetrieveResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestKnowledgeBases:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Zavudev) -> None:
        knowledge_base = client.senders.agent.knowledge_bases.create(
            sender_id="senderId",
            name="Product FAQ",
        )
        assert_matches_type(KnowledgeBaseCreateResponse, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Zavudev) -> None:
        knowledge_base = client.senders.agent.knowledge_bases.create(
            sender_id="senderId",
            name="Product FAQ",
            description="Frequently asked questions about our products",
        )
        assert_matches_type(KnowledgeBaseCreateResponse, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Zavudev) -> None:
        response = client.senders.agent.knowledge_bases.with_raw_response.create(
            sender_id="senderId",
            name="Product FAQ",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base = response.parse()
        assert_matches_type(KnowledgeBaseCreateResponse, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Zavudev) -> None:
        with client.senders.agent.knowledge_bases.with_streaming_response.create(
            sender_id="senderId",
            name="Product FAQ",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base = response.parse()
            assert_matches_type(KnowledgeBaseCreateResponse, knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_create(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            client.senders.agent.knowledge_bases.with_raw_response.create(
                sender_id="",
                name="Product FAQ",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Zavudev) -> None:
        knowledge_base = client.senders.agent.knowledge_bases.retrieve(
            kb_id="kbId",
            sender_id="senderId",
        )
        assert_matches_type(KnowledgeBaseRetrieveResponse, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Zavudev) -> None:
        response = client.senders.agent.knowledge_bases.with_raw_response.retrieve(
            kb_id="kbId",
            sender_id="senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base = response.parse()
        assert_matches_type(KnowledgeBaseRetrieveResponse, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Zavudev) -> None:
        with client.senders.agent.knowledge_bases.with_streaming_response.retrieve(
            kb_id="kbId",
            sender_id="senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base = response.parse()
            assert_matches_type(KnowledgeBaseRetrieveResponse, knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            client.senders.agent.knowledge_bases.with_raw_response.retrieve(
                kb_id="kbId",
                sender_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `kb_id` but received ''"):
            client.senders.agent.knowledge_bases.with_raw_response.retrieve(
                kb_id="",
                sender_id="senderId",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Zavudev) -> None:
        knowledge_base = client.senders.agent.knowledge_bases.update(
            kb_id="kbId",
            sender_id="senderId",
        )
        assert_matches_type(KnowledgeBaseUpdateResponse, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Zavudev) -> None:
        knowledge_base = client.senders.agent.knowledge_bases.update(
            kb_id="kbId",
            sender_id="senderId",
            description="description",
            name="name",
        )
        assert_matches_type(KnowledgeBaseUpdateResponse, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Zavudev) -> None:
        response = client.senders.agent.knowledge_bases.with_raw_response.update(
            kb_id="kbId",
            sender_id="senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base = response.parse()
        assert_matches_type(KnowledgeBaseUpdateResponse, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Zavudev) -> None:
        with client.senders.agent.knowledge_bases.with_streaming_response.update(
            kb_id="kbId",
            sender_id="senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base = response.parse()
            assert_matches_type(KnowledgeBaseUpdateResponse, knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            client.senders.agent.knowledge_bases.with_raw_response.update(
                kb_id="kbId",
                sender_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `kb_id` but received ''"):
            client.senders.agent.knowledge_bases.with_raw_response.update(
                kb_id="",
                sender_id="senderId",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Zavudev) -> None:
        knowledge_base = client.senders.agent.knowledge_bases.list(
            sender_id="senderId",
        )
        assert_matches_type(SyncCursor[AgentKnowledgeBase], knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Zavudev) -> None:
        knowledge_base = client.senders.agent.knowledge_bases.list(
            sender_id="senderId",
            cursor="cursor",
            limit=100,
        )
        assert_matches_type(SyncCursor[AgentKnowledgeBase], knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Zavudev) -> None:
        response = client.senders.agent.knowledge_bases.with_raw_response.list(
            sender_id="senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base = response.parse()
        assert_matches_type(SyncCursor[AgentKnowledgeBase], knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Zavudev) -> None:
        with client.senders.agent.knowledge_bases.with_streaming_response.list(
            sender_id="senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base = response.parse()
            assert_matches_type(SyncCursor[AgentKnowledgeBase], knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_list(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            client.senders.agent.knowledge_bases.with_raw_response.list(
                sender_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Zavudev) -> None:
        knowledge_base = client.senders.agent.knowledge_bases.delete(
            kb_id="kbId",
            sender_id="senderId",
        )
        assert knowledge_base is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Zavudev) -> None:
        response = client.senders.agent.knowledge_bases.with_raw_response.delete(
            kb_id="kbId",
            sender_id="senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base = response.parse()
        assert knowledge_base is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Zavudev) -> None:
        with client.senders.agent.knowledge_bases.with_streaming_response.delete(
            kb_id="kbId",
            sender_id="senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base = response.parse()
            assert knowledge_base is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            client.senders.agent.knowledge_bases.with_raw_response.delete(
                kb_id="kbId",
                sender_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `kb_id` but received ''"):
            client.senders.agent.knowledge_bases.with_raw_response.delete(
                kb_id="",
                sender_id="senderId",
            )


class TestAsyncKnowledgeBases:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncZavudev) -> None:
        knowledge_base = await async_client.senders.agent.knowledge_bases.create(
            sender_id="senderId",
            name="Product FAQ",
        )
        assert_matches_type(KnowledgeBaseCreateResponse, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncZavudev) -> None:
        knowledge_base = await async_client.senders.agent.knowledge_bases.create(
            sender_id="senderId",
            name="Product FAQ",
            description="Frequently asked questions about our products",
        )
        assert_matches_type(KnowledgeBaseCreateResponse, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncZavudev) -> None:
        response = await async_client.senders.agent.knowledge_bases.with_raw_response.create(
            sender_id="senderId",
            name="Product FAQ",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base = await response.parse()
        assert_matches_type(KnowledgeBaseCreateResponse, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncZavudev) -> None:
        async with async_client.senders.agent.knowledge_bases.with_streaming_response.create(
            sender_id="senderId",
            name="Product FAQ",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base = await response.parse()
            assert_matches_type(KnowledgeBaseCreateResponse, knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_create(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            await async_client.senders.agent.knowledge_bases.with_raw_response.create(
                sender_id="",
                name="Product FAQ",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncZavudev) -> None:
        knowledge_base = await async_client.senders.agent.knowledge_bases.retrieve(
            kb_id="kbId",
            sender_id="senderId",
        )
        assert_matches_type(KnowledgeBaseRetrieveResponse, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncZavudev) -> None:
        response = await async_client.senders.agent.knowledge_bases.with_raw_response.retrieve(
            kb_id="kbId",
            sender_id="senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base = await response.parse()
        assert_matches_type(KnowledgeBaseRetrieveResponse, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncZavudev) -> None:
        async with async_client.senders.agent.knowledge_bases.with_streaming_response.retrieve(
            kb_id="kbId",
            sender_id="senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base = await response.parse()
            assert_matches_type(KnowledgeBaseRetrieveResponse, knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            await async_client.senders.agent.knowledge_bases.with_raw_response.retrieve(
                kb_id="kbId",
                sender_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `kb_id` but received ''"):
            await async_client.senders.agent.knowledge_bases.with_raw_response.retrieve(
                kb_id="",
                sender_id="senderId",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncZavudev) -> None:
        knowledge_base = await async_client.senders.agent.knowledge_bases.update(
            kb_id="kbId",
            sender_id="senderId",
        )
        assert_matches_type(KnowledgeBaseUpdateResponse, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncZavudev) -> None:
        knowledge_base = await async_client.senders.agent.knowledge_bases.update(
            kb_id="kbId",
            sender_id="senderId",
            description="description",
            name="name",
        )
        assert_matches_type(KnowledgeBaseUpdateResponse, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncZavudev) -> None:
        response = await async_client.senders.agent.knowledge_bases.with_raw_response.update(
            kb_id="kbId",
            sender_id="senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base = await response.parse()
        assert_matches_type(KnowledgeBaseUpdateResponse, knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncZavudev) -> None:
        async with async_client.senders.agent.knowledge_bases.with_streaming_response.update(
            kb_id="kbId",
            sender_id="senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base = await response.parse()
            assert_matches_type(KnowledgeBaseUpdateResponse, knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            await async_client.senders.agent.knowledge_bases.with_raw_response.update(
                kb_id="kbId",
                sender_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `kb_id` but received ''"):
            await async_client.senders.agent.knowledge_bases.with_raw_response.update(
                kb_id="",
                sender_id="senderId",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncZavudev) -> None:
        knowledge_base = await async_client.senders.agent.knowledge_bases.list(
            sender_id="senderId",
        )
        assert_matches_type(AsyncCursor[AgentKnowledgeBase], knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncZavudev) -> None:
        knowledge_base = await async_client.senders.agent.knowledge_bases.list(
            sender_id="senderId",
            cursor="cursor",
            limit=100,
        )
        assert_matches_type(AsyncCursor[AgentKnowledgeBase], knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncZavudev) -> None:
        response = await async_client.senders.agent.knowledge_bases.with_raw_response.list(
            sender_id="senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base = await response.parse()
        assert_matches_type(AsyncCursor[AgentKnowledgeBase], knowledge_base, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncZavudev) -> None:
        async with async_client.senders.agent.knowledge_bases.with_streaming_response.list(
            sender_id="senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base = await response.parse()
            assert_matches_type(AsyncCursor[AgentKnowledgeBase], knowledge_base, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_list(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            await async_client.senders.agent.knowledge_bases.with_raw_response.list(
                sender_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncZavudev) -> None:
        knowledge_base = await async_client.senders.agent.knowledge_bases.delete(
            kb_id="kbId",
            sender_id="senderId",
        )
        assert knowledge_base is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncZavudev) -> None:
        response = await async_client.senders.agent.knowledge_bases.with_raw_response.delete(
            kb_id="kbId",
            sender_id="senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        knowledge_base = await response.parse()
        assert knowledge_base is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncZavudev) -> None:
        async with async_client.senders.agent.knowledge_bases.with_streaming_response.delete(
            kb_id="kbId",
            sender_id="senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            knowledge_base = await response.parse()
            assert knowledge_base is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            await async_client.senders.agent.knowledge_bases.with_raw_response.delete(
                kb_id="kbId",
                sender_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `kb_id` but received ''"):
            await async_client.senders.agent.knowledge_bases.with_raw_response.delete(
                kb_id="",
                sender_id="senderId",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from zavudev import Zavudev, AsyncZavudev
from tests.utils import assert_matches_type
from zavudev.pagination import SyncCursor, AsyncCursor
from zavudev.types.senders.agent import (
    AgentTool,
    ToolTestResponse,
    ToolCreateResponse,
    ToolUpdateResponse,
    ToolRetrieveResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestTools:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Zavudev) -> None:
        tool = client.senders.agent.tools.create(
            sender_id="senderId",
            description="Get the status of a customer order",
            name="get_order_status",
            parameters={
                "properties": {"order_id": {}},
                "required": ["order_id"],
                "type": "object",
            },
            webhook_url="https://api.example.com/webhooks/order-status",
        )
        assert_matches_type(ToolCreateResponse, tool, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Zavudev) -> None:
        tool = client.senders.agent.tools.create(
            sender_id="senderId",
            description="Get the status of a customer order",
            name="get_order_status",
            parameters={
                "properties": {
                    "order_id": {
                        "description": "The order ID to look up",
                        "type": "string",
                    }
                },
                "required": ["order_id"],
                "type": "object",
            },
            webhook_url="https://api.example.com/webhooks/order-status",
            enabled=True,
            webhook_secret="whsec_...",
        )
        assert_matches_type(ToolCreateResponse, tool, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Zavudev) -> None:
        response = client.senders.agent.tools.with_raw_response.create(
            sender_id="senderId",
            description="Get the status of a customer order",
            name="get_order_status",
            parameters={
                "properties": {"order_id": {}},
                "required": ["order_id"],
                "type": "object",
            },
            webhook_url="https://api.example.com/webhooks/order-status",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tool = response.parse()
        assert_matches_type(ToolCreateResponse, tool, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Zavudev) -> None:
        with client.senders.agent.tools.with_streaming_response.create(
            sender_id="senderId",
            description="Get the status of a customer order",
            name="get_order_status",
            parameters={
                "properties": {"order_id": {}},
                "required": ["order_id"],
                "type": "object",
            },
            webhook_url="https://api.example.com/webhooks/order-status",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tool = response.parse()
            assert_matches_type(ToolCreateResponse, tool, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_create(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            client.senders.agent.tools.with_raw_response.create(
                sender_id="",
                description="Get the status of a customer order",
                name="get_order_status",
                parameters={
                    "properties": {"order_id": {}},
                    "required": ["order_id"],
                    "type": "object",
                },
                webhook_url="https://api.example.com/webhooks/order-status",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Zavudev) -> None:
        tool = client.senders.agent.tools.retrieve(
            tool_id="toolId",
            sender_id="senderId",
        )
        assert_matches_type(ToolRetrieveResponse, tool, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Zavudev) -> None:
        response = client.senders.agent.tools.with_raw_response.retrieve(
            tool_id="toolId",
            sender_id="senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tool = response.parse()
        assert_matches_type(ToolRetrieveResponse, tool, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Zavudev) -> None:
        with client.senders.agent.tools.with_streaming_response.retrieve(
            tool_id="toolId",
            sender_id="senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tool = response.parse()
            assert_matches_type(ToolRetrieveResponse, tool, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            client.senders.agent.tools.with_raw_response.retrieve(
                tool_id="toolId",
                sender_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `tool_id` but received ''"):
            client.senders.agent.tools.with_raw_response.retrieve(
                tool_id="",
                sender_id="senderId",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Zavudev) -> None:
        tool = client.senders.agent.tools.update(
            tool_id="toolId",
            sender_id="senderId",
        )
        assert_matches_type(ToolUpdateResponse, tool, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Zavudev) -> None:
        tool = client.senders.agent.tools.update(
            tool_id="toolId",
            sender_id="senderId",
            description="description",
            enabled=True,
            name="name",
            parameters={
                "properties": {
                    "foo": {
                        "description": "description",
                        "type": "type",
                    }
                },
                "required": ["string"],
                "type": "object",
            },
            webhook_secret="webhookSecret",
            webhook_url="https://example.com",
        )
        assert_matches_type(ToolUpdateResponse, tool, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Zavudev) -> None:
        response = client.senders.agent.tools.with_raw_response.update(
            tool_id="toolId",
            sender_id="senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tool = response.parse()
        assert_matches_type(ToolUpdateResponse, tool, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Zavudev) -> None:
        with client.senders.agent.tools.with_streaming_response.update(
            tool_id="toolId",
            sender_id="senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tool = response.parse()
            assert_matches_type(ToolUpdateResponse, tool, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            client.senders.agent.tools.with_raw_response.update(
                tool_id="toolId",
                sender_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `tool_id` but received ''"):
            client.senders.agent.tools.with_raw_response.update(
                tool_id="",
                sender_id="senderId",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Zavudev) -> None:
        tool = client.senders.agent.tools.list(
            sender_id="senderId",
        )
        assert_matches_type(SyncCursor[AgentTool], tool, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Zavudev) -> None:
        tool = client.senders.agent.tools.list(
            sender_id="senderId",
            cursor="cursor",
            enabled=True,
            limit=100,
        )
        assert_matches_type(SyncCursor[AgentTool], tool, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Zavudev) -> None:
        response = client.senders.agent.tools.with_raw_response.list(
            sender_id="senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tool = response.parse()
        assert_matches_type(SyncCursor[AgentTool], tool, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Zavudev) -> None:
        with client.senders.agent.tools.with_streaming_response.list(
            sender_id="senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tool = response.parse()
            assert_matches_type(SyncCursor[AgentTool], tool, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_list(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            client.senders.agent.tools.with_raw_response.list(
                sender_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Zavudev) -> None:
        tool = client.senders.agent.tools.delete(
            tool_id="toolId",
            sender_id="senderId",
        )
        assert tool is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Zavudev) -> None:
        response = client.senders.agent.tools.with_raw_response.delete(
            tool_id="toolId",
            sender_id="senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tool = response.parse()
        assert tool is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Zavudev) -> None:
        with client.senders.agent.tools.with_streaming_response.delete(
            tool_id="toolId",
            sender_id="senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tool = response.parse()
            assert tool is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            client.senders.agent.tools.with_raw_response.delete(
                tool_id="toolId",
                sender_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `tool_id` but received ''"):
            client.senders.agent.tools.with_raw_response.delete(
                tool_id="",
                sender_id="senderId",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_test(self, client: Zavudev) -> None:
        tool = client.senders.agent.tools.test(
            tool_id="toolId",
            sender_id="senderId",
            test_params={"order_id": "bar"},
        )
        assert_matches_type(ToolTestResponse, tool, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_test(self, client: Zavudev) -> None:
        response = client.senders.agent.tools.with_raw_response.test(
            tool_id="toolId",
            sender_id="senderId",
            test_params={"order_id": "bar"},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tool = response.parse()
        assert_matches_type(ToolTestResponse, tool, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_test(self, client: Zavudev) -> None:
        with client.senders.agent.tools.with_streaming_response.test(
            tool_id="toolId",
            sender_id="senderId",
            test_params={"order_id": "bar"},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tool = response.parse()
            assert_matches_type(ToolTestResponse, tool, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_test(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            client.senders.agent.tools.with_raw_response.test(
                tool_id="toolId",
                sender_id="",
                test_params={"order_id": "bar"},
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `tool_id` but received ''"):
            client.senders.agent.tools.with_raw_response.test(
                tool_id="",
                sender_id="senderId",
                test_params={"order_id": "bar"},
            )


class TestAsyncTools:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncZavudev) -> None:
        tool = await async_client.senders.agent.tools.create(
            sender_id="senderId",
            description="Get the status of a customer order",
            name="get_order_status",
            parameters={
                "properties": {"order_id": {}},
                "required": ["order_id"],
                "type": "object",
            },
            webhook_url="https://api.example.com/webhooks/order-status",
        )
        assert_matches_type(ToolCreateResponse, tool, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncZavudev) -> None:
        tool = await async_client.senders.agent.tools.create(
            sender_id="senderId",
            description="Get the status of a customer order",
            name="get_order_status",
            parameters={
                "properties": {
                    "order_id": {
                        "description": "The order ID to look up",
                        "type": "string",
                    }
                },
                "required": ["order_id"],
                "type": "object",
            },
            webhook_url="https://api.example.com/webhooks/order-status",
            enabled=True,
            webhook_secret="whsec_...",
        )
        assert_matches_type(ToolCreateResponse, tool, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncZavudev) -> None:
        response = await async_client.senders.agent.tools.with_raw_response.create(
            sender_id="senderId",
            description="Get the status of a customer order",
            name="get_order_status",
            parameters={
                "properties": {"order_id": {}},
                "required": ["order_id"],
                "type": "object",
            },
            webhook_url="https://api.example.com/webhooks/order-status",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tool = await response.parse()
        assert_matches_type(ToolCreateResponse, tool, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncZavudev) -> None:
        async with async_client.senders.agent.tools.with_streaming_response.create(
            sender_id="senderId",
            description="Get the status of a customer order",
            name="get_order_status",
            parameters={
                "properties": {"order_id": {}},
                "required": ["order_id"],
                "type": "object",
            },
            webhook_url="https://api.example.com/webhooks/order-status",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tool = await response.parse()
            assert_matches_type(ToolCreateResponse, tool, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_create(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            await async_client.senders.agent.tools.with_raw_response.create(
                sender_id="",
                description="Get the status of a customer order",
                name="get_order_status",
                parameters={
                    "properties": {"order_id": {}},
                    "required": ["order_id"],
                    "type": "object",
                },
                webhook_url="https://api.example.com/webhooks/order-status",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncZavudev) -> None:
        tool = await async_client.senders.agent.tools.retrieve(
            tool_id="toolId",
            sender_id="senderId",
        )
        assert_matches_type(ToolRetrieveResponse, tool, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncZavudev) -> None:
        response = await async_client.senders.agent.tools.with_raw_response.retrieve(
            tool_id="toolId",
            sender_id="senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tool = await response.parse()
        assert_matches_type(ToolRetrieveResponse, tool, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncZavudev) -> None:
        async with async_client.senders.agent.tools.with_streaming_response.retrieve(
            tool_id="toolId",
            sender_id="senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tool = await response.parse()
            assert_matches_type(ToolRetrieveResponse, tool, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            await async_client.senders.agent.tools.with_raw_response.retrieve(
                tool_id="toolId",
                sender_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `tool_id` but received ''"):
            await async_client.senders.agent.tools.with_raw_response.retrieve(
                tool_id="",
                sender_id="senderId",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncZavudev) -> None:
        tool = await async_client.senders.agent.tools.update(
            tool_id="toolId",
            sender_id="senderId",
        )
        assert_matches_type(ToolUpdateResponse, tool, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncZavudev) -> None:
        tool = await async_client.senders.agent.tools.update(
            tool_id="toolId",
            sender_id="senderId",
            description="description",
            enabled=True,
            name="name",
            parameters={
                "properties": {
                    "foo": {
                        "description": "description",
                        "type": "type",
                    }
                },
                "required": ["string"],
                "type": "object",
            },
            webhook_secret="webhookSecret",
            webhook_url="https://example.com",
        )
        assert_matches_type(ToolUpdateResponse, tool, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncZavudev) -> None:
        response = await async_client.senders.agent.tools.with_raw_response.update(
            tool_id="toolId",
            sender_id="senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tool = await response.parse()
        assert_matches_type(ToolUpdateResponse, tool, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncZavudev) -> None:
        async with async_client.senders.agent.tools.with_streaming_response.update(
            tool_id="toolId",
            sender_id="senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tool = await response.parse()
            assert_matches_type(ToolUpdateResponse, tool, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            await async_client.senders.agent.tools.with_raw_response.update(
                tool_id="toolId",
                sender_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `tool_id` but received ''"):
            await async_client.senders.agent.tools.with_raw_response.update(
                tool_id="",
                sender_id="senderId",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncZavudev) -> None:
        tool = await async_client.senders.agent.tools.list(
            sender_id="senderId",
        )
        assert_matches_type(AsyncCursor[AgentTool], tool, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncZavudev) -> None:
        tool = await async_client.senders.agent.tools.list(
            sender_id="senderId",
            cursor="cursor",
            enabled=True,
            limit=100,
        )
        assert_matches_type(AsyncCursor[AgentTool], tool, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncZavudev) -> None:
        response = await async_client.senders.agent.tools.with_raw_response.list(
            sender_id="senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tool = await response.parse()
        assert_matches_type(AsyncCursor[AgentTool], tool, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncZavudev) -> None:
        async with async_client.senders.agent.tools.with_streaming_response.list(
            sender_id="senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tool = await response.parse()
            assert_matches_type(AsyncCursor[AgentTool], tool, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_list(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            await async_client.senders.agent.tools.with_raw_response.list(
                sender_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncZavudev) -> None:
        tool = await async_client.senders.agent.tools.delete(
            tool_id="toolId",
            sender_id="senderId",
        )
        assert tool is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncZavudev) -> None:
        response = await async_client.senders.agent.tools.with_raw_response.delete(
            tool_id="toolId",
            sender_id="senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tool = await response.parse()
        assert tool is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncZavudev) -> None:
        async with async_client.senders.agent.tools.with_streaming_response.delete(
            tool_id="toolId",
            sender_id="senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tool = await response.parse()
            assert tool is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            await async_client.senders.agent.tools.with_raw_response.delete(
                tool_id="toolId",
                sender_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `tool_id` but received ''"):
            await async_client.senders.agent.tools.with_raw_response.delete(
                tool_id="",
                sender_id="senderId",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_test(self, async_client: AsyncZavudev) -> None:
        tool = await async_client.senders.agent.tools.test(
            tool_id="toolId",
            sender_id="senderId",
            test_params={"order_id": "bar"},
        )
        assert_matches_type(ToolTestResponse, tool, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_test(self, async_client: AsyncZavudev) -> None:
        response = await async_client.senders.agent.tools.with_raw_response.test(
            tool_id="toolId",
            sender_id="senderId",
            test_params={"order_id": "bar"},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tool = await response.parse()
        assert_matches_type(ToolTestResponse, tool, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_test(self, async_client: AsyncZavudev) -> None:
        async with async_client.senders.agent.tools.with_streaming_response.test(
            tool_id="toolId",
            sender_id="senderId",
            test_params={"order_id": "bar"},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tool = await response.parse()
            assert_matches_type(ToolTestResponse, tool, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_test(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            await async_client.senders.agent.tools.with_raw_response.test(
                tool_id="toolId",
                sender_id="",
                test_params={"order_id": "bar"},
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `tool_id` but received ''"):
            await async_client.senders.agent.tools.with_raw_response.test(
                tool_id="",
                sender_id="senderId",
                test_params={"order_id": "bar"},
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

from .._types import SequenceNotStr
from .whatsapp_business_profile_vertical import WhatsappBusinessProfileVertical

__all__ = ["SenderUpdateProfileParams"]


class SenderUpdateProfileParams(TypedDict, total=False):
    about: str
    """Short description of the business (max 139 characters)."""

    address: str
    """Physical address of the business (max 256 characters)."""

    description: str
    """Extended description of the business (max 512 characters)."""

    email: str
    """Business email address."""

    vertical: WhatsappBusinessProfileVertical
    """Business category for WhatsApp Business profile."""

    websites: SequenceNotStr[str]
    """Business website URLs (maximum 2)."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["MessageReactParams"]


class MessageReactParams(TypedDict, total=False):
    emoji: Required[str]
    """Single emoji character to react with."""

    zavu_sender: Annotated[str, PropertyInfo(alias="Zavu-Sender")]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["BroadcastContent"]


class BroadcastContent(BaseModel):
    """Content for non-text broadcast message types."""

    filename: Optional[str] = None
    """Filename for documents."""

    media_id: Optional[str] = FieldInfo(alias="mediaId", default=None)
    """Media ID if already uploaded."""

    media_url: Optional[str] = FieldInfo(alias="mediaUrl", default=None)
    """URL of the media file."""

    mime_type: Optional[str] = FieldInfo(alias="mimeType", default=None)
    """MIME type of the media."""

    template_id: Optional[str] = FieldInfo(alias="templateId", default=None)
    """Template ID for template messages."""

    template_variables: Optional[Dict[str, str]] = FieldInfo(alias="templateVariables", default=None)
    """Default template variables (can be overridden per contact)."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from ...._models import BaseModel
from ..agent_provider import AgentProvider

__all__ = ["Agent", "Stats"]


class Stats(BaseModel):
    total_cost: Optional[float] = FieldInfo(alias="totalCost", default=None)
    """Total cost in USD."""

    total_invocations: Optional[int] = FieldInfo(alias="totalInvocations", default=None)

    total_tokens_used: Optional[int] = FieldInfo(alias="totalTokensUsed", default=None)


class Agent(BaseModel):
    """AI Agent configuration for a sender."""

    id: str

    created_at: datetime = FieldInfo(alias="createdAt")

    enabled: bool
    """Whether the agent is active."""

    model: str
    """Model ID (e.g., gpt-4o-mini, claude-3-5-sonnet)."""

    name: str

    provider: AgentProvider
    """LLM provider for the AI agent."""

    sender_id: str = FieldInfo(alias="senderId")

    system_prompt: str = FieldInfo(alias="systemPrompt")
    """System prompt for the agent."""

    updated_at: datetime = FieldInfo(alias="updatedAt")

    context_window_messages: Optional[int] = FieldInfo(alias="contextWindowMessages", default=None)
    """Number of previous messages to include as context."""

    include_contact_metadata: Optional[bool] = FieldInfo(alias="includeContactMetadata", default=None)
    """Whether to include contact metadata in context."""

    max_tokens: Optional[int] = FieldInfo(alias="maxTokens", default=None)
    """Maximum tokens for LLM response."""

    stats: Optional[Stats] = None

    temperature: Optional[float] = None
    """LLM temperature (0-2)."""

    trigger_on_channels: Optional[List[str]] = FieldInfo(alias="triggerOnChannels", default=None)
    """Channels that trigger the agent."""

    trigger_on_message_types: Optional[List[str]] = FieldInfo(alias="triggerOnMessageTypes", default=None)
    """Message types that trigger the agent."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import introspect_validate_phone_params
from .._types import Body, Query, Headers, NotGiven, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.introspect_validate_phone_response import IntrospectValidatePhoneResponse

__all__ = ["IntrospectResource", "AsyncIntrospectResource"]


class IntrospectResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> IntrospectResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/zavudev/sdk-python#accessing-raw-response-data-eg-headers
        """
        return IntrospectResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> IntrospectResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/zavudev/sdk-python#with_streaming_response
        """
        return IntrospectResourceWithStreamingResponse(self)

    def validate_phone(
        self,
        *,
        phone_number: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IntrospectValidatePhoneResponse:
        """
        Validate a phone number and check if a WhatsApp conversation window is open.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/introspect/phone",
            body=maybe_transform(
                {"phone_number": phone_number}, introspect_validate_phone_params.IntrospectValidatePhoneParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IntrospectValidatePhoneResponse,
        )


class AsyncIntrospectResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncIntrospectResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/zavudev/sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncIntrospectResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncIntrospectResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/zavudev/sdk-python#with_streaming_response
        """
        return AsyncIntrospectResourceWithStreamingResponse(self)

    async def validate_phone(
        self,
        *,
        phone_number: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IntrospectValidatePhoneResponse:
        """
        Validate a phone number and check if a WhatsApp conversation window is open.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/introspect/phone",
            body=await async_maybe_transform(
                {"phone_number": phone_number}, introspect_validate_phone_params.IntrospectValidatePhoneParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IntrospectValidatePhoneResponse,
        )


class IntrospectResourceWithRawResponse:
    def __init__(self, introspect: IntrospectResource) -> None:
        self._introspect = introspect

        self.validate_phone = to_raw_response_wrapper(
            introspect.validate_phone,
        )


class AsyncIntrospectResourceWithRawResponse:
    def __init__(self, introspect: AsyncIntrospectResource) -> None:
        self._introspect = introspect

        self.validate_phone = async_to_raw_response_wrapper(
            introspect.validate_phone,
        )


class IntrospectResourceWithStreamingResponse:
    def __init__(self, introspect: IntrospectResource) -> None:
        self._introspect = introspect

        self.validate_phone = to_streamed_response_wrapper(
            introspect.validate_phone,
        )


class AsyncIntrospectResourceWithStreamingResponse:
    def __init__(self, introspect: AsyncIntrospectResource) -> None:
        self._introspect = introspect

        self.validate_phone = async_to_streamed_response_wrapper(
            introspect.validate_phone,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .documents import (
    DocumentsResource,
    AsyncDocumentsResource,
    DocumentsResourceWithRawResponse,
    AsyncDocumentsResourceWithRawResponse,
    DocumentsResourceWithStreamingResponse,
    AsyncDocumentsResourceWithStreamingResponse,
)
from .knowledge_bases import (
    KnowledgeBasesResource,
    AsyncKnowledgeBasesResource,
    KnowledgeBasesResourceWithRawResponse,
    AsyncKnowledgeBasesResourceWithRawResponse,
    KnowledgeBasesResourceWithStreamingResponse,
    AsyncKnowledgeBasesResourceWithStreamingResponse,
)

__all__ = [
    "DocumentsResource",
    "AsyncDocumentsResource",
    "DocumentsResourceWithRawResponse",
    "AsyncDocumentsResourceWithRawResponse",
    "DocumentsResourceWithStreamingResponse",
    "AsyncDocumentsResourceWithStreamingResponse",
    "KnowledgeBasesResource",
    "AsyncKnowledgeBasesResource",
    "KnowledgeBasesResourceWithRawResponse",
    "AsyncKnowledgeBasesResourceWithRawResponse",
    "KnowledgeBasesResourceWithStreamingResponse",
    "AsyncKnowledgeBasesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from zavudev import Zavudev, AsyncZavudev
from tests.utils import assert_matches_type
from zavudev.types import (
    OwnedPhoneNumber,
    PhoneNumberUpdateResponse,
    PhoneNumberPurchaseResponse,
    PhoneNumberRetrieveResponse,
    PhoneNumberRequirementsResponse,
    PhoneNumberSearchAvailableResponse,
)
from zavudev.pagination import SyncCursor, AsyncCursor

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestPhoneNumbers:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Zavudev) -> None:
        phone_number = client.phone_numbers.retrieve(
            "phoneNumberId",
        )
        assert_matches_type(PhoneNumberRetrieveResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Zavudev) -> None:
        response = client.phone_numbers.with_raw_response.retrieve(
            "phoneNumberId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number = response.parse()
        assert_matches_type(PhoneNumberRetrieveResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Zavudev) -> None:
        with client.phone_numbers.with_streaming_response.retrieve(
            "phoneNumberId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number = response.parse()
            assert_matches_type(PhoneNumberRetrieveResponse, phone_number, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `phone_number_id` but received ''"):
            client.phone_numbers.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Zavudev) -> None:
        phone_number = client.phone_numbers.update(
            phone_number_id="phoneNumberId",
        )
        assert_matches_type(PhoneNumberUpdateResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Zavudev) -> None:
        phone_number = client.phone_numbers.update(
            phone_number_id="phoneNumberId",
            name="Support Line",
            sender_id="senderId",
        )
        assert_matches_type(PhoneNumberUpdateResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Zavudev) -> None:
        response = client.phone_numbers.with_raw_response.update(
            phone_number_id="phoneNumberId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number = response.parse()
        assert_matches_type(PhoneNumberUpdateResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Zavudev) -> None:
        with client.phone_numbers.with_streaming_response.update(
            phone_number_id="phoneNumberId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number = response.parse()
            assert_matches_type(PhoneNumberUpdateResponse, phone_number, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `phone_number_id` but received ''"):
            client.phone_numbers.with_raw_response.update(
                phone_number_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Zavudev) -> None:
        phone_number = client.phone_numbers.list()
        assert_matches_type(SyncCursor[OwnedPhoneNumber], phone_number, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Zavudev) -> None:
        phone_number = client.phone_numbers.list(
            cursor="cursor",
            limit=100,
            status="active",
        )
        assert_matches_type(SyncCursor[OwnedPhoneNumber], phone_number, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Zavudev) -> None:
        response = client.phone_numbers.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number = response.parse()
        assert_matches_type(SyncCursor[OwnedPhoneNumber], phone_number, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Zavudev) -> None:
        with client.phone_numbers.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number = response.parse()
            assert_matches_type(SyncCursor[OwnedPhoneNumber], phone_number, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_purchase(self, client: Zavudev) -> None:
        phone_number = client.phone_numbers.purchase(
            phone_number="+15551234567",
        )
        assert_matches_type(PhoneNumberPurchaseResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_purchase_with_all_params(self, client: Zavudev) -> None:
        phone_number = client.phone_numbers.purchase(
            phone_number="+15551234567",
            name="Primary Line",
        )
        assert_matches_type(PhoneNumberPurchaseResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_purchase(self, client: Zavudev) -> None:
        response = client.phone_numbers.with_raw_response.purchase(
            phone_number="+15551234567",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number = response.parse()
        assert_matches_type(PhoneNumberPurchaseResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_purchase(self, client: Zavudev) -> None:
        with client.phone_numbers.with_streaming_response.purchase(
            phone_number="+15551234567",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number = response.parse()
            assert_matches_type(PhoneNumberPurchaseResponse, phone_number, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_release(self, client: Zavudev) -> None:
        phone_number = client.phone_numbers.release(
            "phoneNumberId",
        )
        assert phone_number is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_release(self, client: Zavudev) -> None:
        response = client.phone_numbers.with_raw_response.release(
            "phoneNumberId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number = response.parse()
        assert phone_number is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_release(self, client: Zavudev) -> None:
        with client.phone_numbers.with_streaming_response.release(
            "phoneNumberId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number = response.parse()
            assert phone_number is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_release(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `phone_number_id` but received ''"):
            client.phone_numbers.with_raw_response.release(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_requirements(self, client: Zavudev) -> None:
        phone_number = client.phone_numbers.requirements(
            country_code="xx",
        )
        assert_matches_type(PhoneNumberRequirementsResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_requirements_with_all_params(self, client: Zavudev) -> None:
        phone_number = client.phone_numbers.requirements(
            country_code="xx",
            type="local",
        )
        assert_matches_type(PhoneNumberRequirementsResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_requirements(self, client: Zavudev) -> None:
        response = client.phone_numbers.with_raw_response.requirements(
            country_code="xx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number = response.parse()
        assert_matches_type(PhoneNumberRequirementsResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_requirements(self, client: Zavudev) -> None:
        with client.phone_numbers.with_streaming_response.requirements(
            country_code="xx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number = response.parse()
            assert_matches_type(PhoneNumberRequirementsResponse, phone_number, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_search_available(self, client: Zavudev) -> None:
        phone_number = client.phone_numbers.search_available(
            country_code="xx",
        )
        assert_matches_type(PhoneNumberSearchAvailableResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_search_available_with_all_params(self, client: Zavudev) -> None:
        phone_number = client.phone_numbers.search_available(
            country_code="xx",
            contains="contains",
            limit=50,
            type="local",
        )
        assert_matches_type(PhoneNumberSearchAvailableResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_search_available(self, client: Zavudev) -> None:
        response = client.phone_numbers.with_raw_response.search_available(
            country_code="xx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number = response.parse()
        assert_matches_type(PhoneNumberSearchAvailableResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_search_available(self, client: Zavudev) -> None:
        with client.phone_numbers.with_streaming_response.search_available(
            country_code="xx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number = response.parse()
            assert_matches_type(PhoneNumberSearchAvailableResponse, phone_number, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncPhoneNumbers:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncZavudev) -> None:
        phone_number = await async_client.phone_numbers.retrieve(
            "phoneNumberId",
        )
        assert_matches_type(PhoneNumberRetrieveResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncZavudev) -> None:
        response = await async_client.phone_numbers.with_raw_response.retrieve(
            "phoneNumberId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number = await response.parse()
        assert_matches_type(PhoneNumberRetrieveResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncZavudev) -> None:
        async with async_client.phone_numbers.with_streaming_response.retrieve(
            "phoneNumberId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number = await response.parse()
            assert_matches_type(PhoneNumberRetrieveResponse, phone_number, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `phone_number_id` but received ''"):
            await async_client.phone_numbers.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncZavudev) -> None:
        phone_number = await async_client.phone_numbers.update(
            phone_number_id="phoneNumberId",
        )
        assert_matches_type(PhoneNumberUpdateResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncZavudev) -> None:
        phone_number = await async_client.phone_numbers.update(
            phone_number_id="phoneNumberId",
            name="Support Line",
            sender_id="senderId",
        )
        assert_matches_type(PhoneNumberUpdateResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncZavudev) -> None:
        response = await async_client.phone_numbers.with_raw_response.update(
            phone_number_id="phoneNumberId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number = await response.parse()
        assert_matches_type(PhoneNumberUpdateResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncZavudev) -> None:
        async with async_client.phone_numbers.with_streaming_response.update(
            phone_number_id="phoneNumberId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number = await response.parse()
            assert_matches_type(PhoneNumberUpdateResponse, phone_number, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `phone_number_id` but received ''"):
            await async_client.phone_numbers.with_raw_response.update(
                phone_number_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncZavudev) -> None:
        phone_number = await async_client.phone_numbers.list()
        assert_matches_type(AsyncCursor[OwnedPhoneNumber], phone_number, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncZavudev) -> None:
        phone_number = await async_client.phone_numbers.list(
            cursor="cursor",
            limit=100,
            status="active",
        )
        assert_matches_type(AsyncCursor[OwnedPhoneNumber], phone_number, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncZavudev) -> None:
        response = await async_client.phone_numbers.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number = await response.parse()
        assert_matches_type(AsyncCursor[OwnedPhoneNumber], phone_number, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncZavudev) -> None:
        async with async_client.phone_numbers.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number = await response.parse()
            assert_matches_type(AsyncCursor[OwnedPhoneNumber], phone_number, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_purchase(self, async_client: AsyncZavudev) -> None:
        phone_number = await async_client.phone_numbers.purchase(
            phone_number="+15551234567",
        )
        assert_matches_type(PhoneNumberPurchaseResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_purchase_with_all_params(self, async_client: AsyncZavudev) -> None:
        phone_number = await async_client.phone_numbers.purchase(
            phone_number="+15551234567",
            name="Primary Line",
        )
        assert_matches_type(PhoneNumberPurchaseResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_purchase(self, async_client: AsyncZavudev) -> None:
        response = await async_client.phone_numbers.with_raw_response.purchase(
            phone_number="+15551234567",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number = await response.parse()
        assert_matches_type(PhoneNumberPurchaseResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_purchase(self, async_client: AsyncZavudev) -> None:
        async with async_client.phone_numbers.with_streaming_response.purchase(
            phone_number="+15551234567",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number = await response.parse()
            assert_matches_type(PhoneNumberPurchaseResponse, phone_number, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_release(self, async_client: AsyncZavudev) -> None:
        phone_number = await async_client.phone_numbers.release(
            "phoneNumberId",
        )
        assert phone_number is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_release(self, async_client: AsyncZavudev) -> None:
        response = await async_client.phone_numbers.with_raw_response.release(
            "phoneNumberId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number = await response.parse()
        assert phone_number is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_release(self, async_client: AsyncZavudev) -> None:
        async with async_client.phone_numbers.with_streaming_response.release(
            "phoneNumberId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number = await response.parse()
            assert phone_number is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_release(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `phone_number_id` but received ''"):
            await async_client.phone_numbers.with_raw_response.release(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_requirements(self, async_client: AsyncZavudev) -> None:
        phone_number = await async_client.phone_numbers.requirements(
            country_code="xx",
        )
        assert_matches_type(PhoneNumberRequirementsResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_requirements_with_all_params(self, async_client: AsyncZavudev) -> None:
        phone_number = await async_client.phone_numbers.requirements(
            country_code="xx",
            type="local",
        )
        assert_matches_type(PhoneNumberRequirementsResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_requirements(self, async_client: AsyncZavudev) -> None:
        response = await async_client.phone_numbers.with_raw_response.requirements(
            country_code="xx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number = await response.parse()
        assert_matches_type(PhoneNumberRequirementsResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_requirements(self, async_client: AsyncZavudev) -> None:
        async with async_client.phone_numbers.with_streaming_response.requirements(
            country_code="xx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number = await response.parse()
            assert_matches_type(PhoneNumberRequirementsResponse, phone_number, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_search_available(self, async_client: AsyncZavudev) -> None:
        phone_number = await async_client.phone_numbers.search_available(
            country_code="xx",
        )
        assert_matches_type(PhoneNumberSearchAvailableResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_search_available_with_all_params(self, async_client: AsyncZavudev) -> None:
        phone_number = await async_client.phone_numbers.search_available(
            country_code="xx",
            contains="contains",
            limit=50,
            type="local",
        )
        assert_matches_type(PhoneNumberSearchAvailableResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_search_available(self, async_client: AsyncZavudev) -> None:
        response = await async_client.phone_numbers.with_raw_response.search_available(
            country_code="xx",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number = await response.parse()
        assert_matches_type(PhoneNumberSearchAvailableResponse, phone_number, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_search_available(self, async_client: AsyncZavudev) -> None:
        async with async_client.phone_numbers.with_streaming_response.search_available(
            country_code="xx",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number = await response.parse()
            assert_matches_type(PhoneNumberSearchAvailableResponse, phone_number, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from zavudev import Zavudev, AsyncZavudev
from tests.utils import assert_matches_type
from zavudev.pagination import SyncCursor, AsyncCursor
from zavudev.types.senders.agent import AgentDocument
from zavudev.types.senders.agent.knowledge_bases import (
    DocumentCreateResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestDocuments:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Zavudev) -> None:
        document = client.senders.agent.knowledge_bases.documents.create(
            kb_id="kbId",
            sender_id="senderId",
            content="Our return policy allows returns within 30 days of purchase...",
            title="Return Policy",
        )
        assert_matches_type(DocumentCreateResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Zavudev) -> None:
        response = client.senders.agent.knowledge_bases.documents.with_raw_response.create(
            kb_id="kbId",
            sender_id="senderId",
            content="Our return policy allows returns within 30 days of purchase...",
            title="Return Policy",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = response.parse()
        assert_matches_type(DocumentCreateResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Zavudev) -> None:
        with client.senders.agent.knowledge_bases.documents.with_streaming_response.create(
            kb_id="kbId",
            sender_id="senderId",
            content="Our return policy allows returns within 30 days of purchase...",
            title="Return Policy",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = response.parse()
            assert_matches_type(DocumentCreateResponse, document, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_create(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            client.senders.agent.knowledge_bases.documents.with_raw_response.create(
                kb_id="kbId",
                sender_id="",
                content="Our return policy allows returns within 30 days of purchase...",
                title="Return Policy",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `kb_id` but received ''"):
            client.senders.agent.knowledge_bases.documents.with_raw_response.create(
                kb_id="",
                sender_id="senderId",
                content="Our return policy allows returns within 30 days of purchase...",
                title="Return Policy",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Zavudev) -> None:
        document = client.senders.agent.knowledge_bases.documents.list(
            kb_id="kbId",
            sender_id="senderId",
        )
        assert_matches_type(SyncCursor[AgentDocument], document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Zavudev) -> None:
        document = client.senders.agent.knowledge_bases.documents.list(
            kb_id="kbId",
            sender_id="senderId",
            cursor="cursor",
            limit=100,
        )
        assert_matches_type(SyncCursor[AgentDocument], document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Zavudev) -> None:
        response = client.senders.agent.knowledge_bases.documents.with_raw_response.list(
            kb_id="kbId",
            sender_id="senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = response.parse()
        assert_matches_type(SyncCursor[AgentDocument], document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Zavudev) -> None:
        with client.senders.agent.knowledge_bases.documents.with_streaming_response.list(
            kb_id="kbId",
            sender_id="senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = response.parse()
            assert_matches_type(SyncCursor[AgentDocument], document, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_list(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            client.senders.agent.knowledge_bases.documents.with_raw_response.list(
                kb_id="kbId",
                sender_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `kb_id` but received ''"):
            client.senders.agent.knowledge_bases.documents.with_raw_response.list(
                kb_id="",
                sender_id="senderId",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Zavudev) -> None:
        document = client.senders.agent.knowledge_bases.documents.delete(
            doc_id="docId",
            sender_id="senderId",
            kb_id="kbId",
        )
        assert document is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Zavudev) -> None:
        response = client.senders.agent.knowledge_bases.documents.with_raw_response.delete(
            doc_id="docId",
            sender_id="senderId",
            kb_id="kbId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = response.parse()
        assert document is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Zavudev) -> None:
        with client.senders.agent.knowledge_bases.documents.with_streaming_response.delete(
            doc_id="docId",
            sender_id="senderId",
            kb_id="kbId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = response.parse()
            assert document is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            client.senders.agent.knowledge_bases.documents.with_raw_response.delete(
                doc_id="docId",
                sender_id="",
                kb_id="kbId",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `kb_id` but received ''"):
            client.senders.agent.knowledge_bases.documents.with_raw_response.delete(
                doc_id="docId",
                sender_id="senderId",
                kb_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `doc_id` but received ''"):
            client.senders.agent.knowledge_bases.documents.with_raw_response.delete(
                doc_id="",
                sender_id="senderId",
                kb_id="kbId",
            )


class TestAsyncDocuments:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncZavudev) -> None:
        document = await async_client.senders.agent.knowledge_bases.documents.create(
            kb_id="kbId",
            sender_id="senderId",
            content="Our return policy allows returns within 30 days of purchase...",
            title="Return Policy",
        )
        assert_matches_type(DocumentCreateResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncZavudev) -> None:
        response = await async_client.senders.agent.knowledge_bases.documents.with_raw_response.create(
            kb_id="kbId",
            sender_id="senderId",
            content="Our return policy allows returns within 30 days of purchase...",
            title="Return Policy",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = await response.parse()
        assert_matches_type(DocumentCreateResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncZavudev) -> None:
        async with async_client.senders.agent.knowledge_bases.documents.with_streaming_response.create(
            kb_id="kbId",
            sender_id="senderId",
            content="Our return policy allows returns within 30 days of purchase...",
            title="Return Policy",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = await response.parse()
            assert_matches_type(DocumentCreateResponse, document, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_create(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            await async_client.senders.agent.knowledge_bases.documents.with_raw_response.create(
                kb_id="kbId",
                sender_id="",
                content="Our return policy allows returns within 30 days of purchase...",
                title="Return Policy",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `kb_id` but received ''"):
            await async_client.senders.agent.knowledge_bases.documents.with_raw_response.create(
                kb_id="",
                sender_id="senderId",
                content="Our return policy allows returns within 30 days of purchase...",
                title="Return Policy",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncZavudev) -> None:
        document = await async_client.senders.agent.knowledge_bases.documents.list(
            kb_id="kbId",
            sender_id="senderId",
        )
        assert_matches_type(AsyncCursor[AgentDocument], document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncZavudev) -> None:
        document = await async_client.senders.agent.knowledge_bases.documents.list(
            kb_id="kbId",
            sender_id="senderId",
            cursor="cursor",
            limit=100,
        )
        assert_matches_type(AsyncCursor[AgentDocument], document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncZavudev) -> None:
        response = await async_client.senders.agent.knowledge_bases.documents.with_raw_response.list(
            kb_id="kbId",
            sender_id="senderId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = await response.parse()
        assert_matches_type(AsyncCursor[AgentDocument], document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncZavudev) -> None:
        async with async_client.senders.agent.knowledge_bases.documents.with_streaming_response.list(
            kb_id="kbId",
            sender_id="senderId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = await response.parse()
            assert_matches_type(AsyncCursor[AgentDocument], document, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_list(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            await async_client.senders.agent.knowledge_bases.documents.with_raw_response.list(
                kb_id="kbId",
                sender_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `kb_id` but received ''"):
            await async_client.senders.agent.knowledge_bases.documents.with_raw_response.list(
                kb_id="",
                sender_id="senderId",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncZavudev) -> None:
        document = await async_client.senders.agent.knowledge_bases.documents.delete(
            doc_id="docId",
            sender_id="senderId",
            kb_id="kbId",
        )
        assert document is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncZavudev) -> None:
        response = await async_client.senders.agent.knowledge_bases.documents.with_raw_response.delete(
            doc_id="docId",
            sender_id="senderId",
            kb_id="kbId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = await response.parse()
        assert document is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncZavudev) -> None:
        async with async_client.senders.agent.knowledge_bases.documents.with_streaming_response.delete(
            doc_id="docId",
            sender_id="senderId",
            kb_id="kbId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = await response.parse()
            assert document is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `sender_id` but received ''"):
            await async_client.senders.agent.knowledge_bases.documents.with_raw_response.delete(
                doc_id="docId",
                sender_id="",
                kb_id="kbId",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `kb_id` but received ''"):
            await async_client.senders.agent.knowledge_bases.documents.with_raw_response.delete(
                doc_id="docId",
                sender_id="senderId",
                kb_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `doc_id` but received ''"):
            await async_client.senders.agent.knowledge_bases.documents.with_raw_response.delete(
                doc_id="",
                sender_id="senderId",
                kb_id="kbId",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ...._models import BaseModel
from .agent_flow import AgentFlow

__all__ = ["FlowRetrieveResponse"]


class FlowRetrieveResponse(BaseModel):
    flow: AgentFlow
